import time
from datetime import datetime

"""----------逻辑函数----------"""


def _get_current_date(_format: str = '%Y-%m-%d %H:%M:%S') -> str:
    """获取当前的标准格式时间
    :param _format: 自定义时间格式
    :return: 标准格式时间
    """
    return time.strftime(_format, time.localtime())


def _format_duration(duration: float, _format: str = '%H:%M:%S') -> str:
    """将一个时长（秒）转换为格式化文本，主要用于计时
    :param duration: 时长，秒
    :param _format: 自定义时间格式
    :return: 时分秒格式的时长"""
    days = int(duration // 86400)
    hours = int(duration // 3600)
    minutes = int((duration % 3600) // 60)
    seconds = int(duration % 60)

    time_str = _format.replace('%D', str(days))
    time_str = time_str.replace('%H', str(hours))
    time_str = time_str.replace('%M', str(minutes))
    time_str = time_str.replace('%S', str(seconds))

    return time_str


def _convert_duration_to_date(tm_seconds: float, _format: str = '%Y-%m-%d %H:%M:%S') -> str:
    """将一个秒数时间转换为标准格式时间
    :param tm_seconds: 时长，秒
    :param _format: 自定义时间格式
    :return: 标准格式时间"""
    date = datetime.fromtimestamp(tm_seconds).strftime(_format)

    return date


"""----------调用函数----------"""


def get_current_time(_format: str = '%Y-%m-%d %H:%M:%S') -> str:
    """获取当前的标准格式时间
    :param _format: 自定义时间格式
    :return: 标准格式时间
    """
    return _get_current_date(_format)


def get_current_date(_format: str = '%Y-%m-%d %H:%M:%S') -> str:
    """获取当前的标准格式时间
    :param _format: 自定义时间格式
    :return: 标准格式时间
    """
    return _get_current_date(_format)


def format_duration(duration: float, _format: str = '%H:%M:%S') -> str:
    """将一个时长（秒）转换为格式化文本，主要用于计时
    :param duration: 时长，秒
    :param _format: 自定义时间格式
    :return: 时分秒格式的时长"""
    return _format_duration(duration, _format)


def convert_time_hms(duration: float, _format: str = '%H:%M:%S') -> str:
    """将一个时长（秒）转换为格式化文本，主要用于计时
    :param duration: 时长，秒
    :param _format: 自定义时间格式
    :return: 时分秒格式的时长"""
    return _format_duration(duration, _format)


def convert_duration_to_date(tm_seconds: float, _format: str = '%Y-%m-%d %H:%M:%S') -> str:
    """将一个秒数时间转换为标准格式时间
    :param tm_seconds: 时长，秒
    :param _format: 自定义时间格式
    :return: 标准格式时间"""
    return convert_duration_to_date(tm_seconds, _format)


def convert_time_ymd(tm_seconds: float, _format: str = '%Y-%m-%d %H:%M:%S') -> str:
    """将一个秒数时间转换为标准格式时间
    :param tm_seconds: 时长，秒
    :param _format: 自定义时间格式
    :return: 标准格式时间"""
    return convert_duration_to_date(tm_seconds, _format)



import ctypes

"""----------逻辑函数----------"""


def _is_file_hidden(path: str) -> bool:
    """文件是否隐藏
    :param path: 文件路径"""
    get_file_attributes_w = ctypes.windll.kernel32.GetFileAttributesW
    file_attribute_hidden = 0x2
    invalid_file_attributes = -1

    def is_hidden(_file):
        # 获取文件属性
        attrs = get_file_attributes_w(_file)
        if attrs == invalid_file_attributes:
            # 文件不存在或无法访问
            return False

        return attrs & file_attribute_hidden == file_attribute_hidden

    return is_hidden(path)


"""----------调用函数----------"""


def is_hidden_file(path: str) -> bool:
    """文件是否隐藏
    :param path: 文件路径"""
    return _is_file_hidden(path)


def is_file_hidden(path: str) -> bool:
    """文件是否隐藏
    :param path: 文件路径"""
    return _is_file_hidden(path)



import subprocess

"""----------逻辑函数----------"""


def _flush_dns():
    """刷新DNS缓存"""
    subprocess.run(['ipconfig', '/flushdns'], shell=True)


"""----------调用函数----------"""


def flush_dns():
    """刷新DNS缓存"""
    _flush_dns()



from typing import Union, Tuple

"""----------逻辑函数----------"""


def _dedup_list(lst: list) -> Tuple[list, list]:
    """"剔除列表中的重复项"""
    list_filter = []  # 去重后的列表
    list_removed = []  # 被去重元素
    for i in lst:
        if i not in list_filter:
            list_filter.append(i)
        else:
            list_removed.append(i)
    return list_filter, list_removed


def _merge_intersection_item(items: Union[list, tuple, set]) -> list:
    """合并有交集的容器，支持集合/列表/元组
    [(1, 2), {2, 3}, (5, 6)]->[(1, 2, 3), (5, 6)]"""
    merged_list = []

    for i in range(len(items)):
        set_merged = False

        for j in range(len(merged_list)):
            if set(items[i]) & set(merged_list[j]):
                merged_list[j] = set(set(items[i]) | set(merged_list[j]))
                set_merged = True
                break

        if not set_merged:
            merged_list.append(items[i])

    return merged_list


"""----------调用函数----------"""


def dedup_list(lst: list) -> list:
    """"剔除列表中的重复项"""
    return _dedup_list(lst)[0]


def merge_intersection_item(items: Union[list, tuple, set]) -> list:
    """合并有交集的容器，支持集合/列表/元组
    [(1, 2), {2, 3}, (5, 6)]->[(1, 2, 3), (5, 6)]"""
    return _merge_intersection_item(items)



import xxhash

"""----------逻辑函数----------"""


def _calc_xxhash_from_bytes(data: bytes) -> str:
    """从字节流计算xxHash"""
    hasher = xxhash.xxh64()
    hasher.update(data)
    return hasher.hexdigest()


def _calc_xxhash_from_file(file_path: str, block_size: int = 65536) -> str:
    """从文件计算xxHash"""
    hasher = xxhash.xxh64()
    with open(file_path, 'rb') as f:
        while chunk := f.read(block_size):
            hasher.update(chunk)
    return hasher.hexdigest()


def _calc_xxhash_from_files(file_paths: list, block_size: int = 65536) -> str:
    """从文件列表计算xxHash"""
    hasher = xxhash.xxh64()
    for file_path in file_paths:
        with open(file_path, 'rb') as f:
            while chunk := f.read(block_size):
                hasher.update(chunk)
    return hasher.hexdigest()


"""----------调用函数----------"""


def calc_xxhash_from_bytes(data: bytes) -> str:
    """从字节流计算xxHash"""
    return _calc_xxhash_from_bytes(data)


def calc_xxhash_from_file(file_path: str, block_size: int = 65536) -> str:
    """从文件计算xxHash"""
    return _calc_xxhash_from_file(file_path, block_size)


def calc_xxhash_from_files(file_paths: list, block_size: int = 65536) -> str:
    """从文件列表计算xxHash"""
    return _calc_xxhash_from_files(file_paths, block_size)



import inspect
import time
from typing import Tuple

"""----------逻辑函数----------"""


def _get_function_info(mode: str = 'current') -> Tuple[str, str, str]:
    """获取当前运行/上个运行的函数的信息
    :param mode: 'current' 或 'last'
    :return: 运行时间，函数名，函数文件路径
    """
    # 获取调用栈
    stack_trace = inspect.stack()  # stack_trace[0]为当前运行函数，stack_trace[1]为调用当前运行的函数

    # 函数基础信息参数
    local_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())  # 调用时间
    function_name = ''  # 函数名
    caller_path = ''  # 函数文件路径

    # 获取函数信息
    if mode.lower() == 'current':
        function_name = stack_trace[1].function
        caller_path = stack_trace[1].filename
    elif mode.lower() == 'last':
        if len(stack_trace) >= 3:
            function_name = stack_trace[2].function
            caller_path = stack_trace[2].filename

    if function_name and caller_path:
        return local_time, function_name, caller_path
    else:
        return None, None, None


def _get_subclasses(cls) -> list:
    """获取所有子类对象
    :return: 子类对象列表"""
    subclasses = []
    for subclass in cls.__subclasses__():
        subclasses.append(subclass)
        subclasses.extend(_get_subclasses(subclass))
    return subclasses


"""----------调用函数----------"""


def print_function_info(mode: str = 'current'):
    """获取当前运行/上个运行的函数的信息
    :param mode: 'current' 或 'last'
    :return: 运行时间，函数名，函数文件路径
    """
    local_time, function_name, caller_path = _get_function_info(mode)
    print(local_time, function_name, caller_path)


def get_function_info(mode: str = 'current') -> Tuple[str, str, str]:
    """获取当前运行/上个运行的函数的信息
    :param mode: 'current' 或 'last'
    :return: 运行时间，函数名，函数文件路径
    """
    return _get_function_info(mode)


def get_subclasses(cls) -> list:
    """获取所有子类对象
    :return: 子类对象列表"""
    return _get_subclasses(cls)



import os
from typing import Tuple

import natsort

"""----------逻辑函数----------"""


def _split_path(path: str) -> Tuple[str, str, str]:
    """拆分路径为父目录路径，文件名（不含文件扩展名），文件扩展名
    :param path: 需要拆分的路径
    :return: 父目录路径，文件名（不含文件扩展名），文件扩展名"""
    if os.path.isfile(path):
        _temp_path, file_extension = os.path.splitext(path)
        parent_dirpath, filetitle = os.path.split(_temp_path)
    elif os.path.isdir(path):
        file_extension = ''
        parent_dirpath, filetitle = os.path.split(path)
    else:
        raise Exception('非法路径')

    return parent_dirpath, filetitle, file_extension


def _reverse_path(path: str) -> str:
    """反转路径层级，从后往前排列目录层级
    :param path: 需要反转的路径
    :return: 反转后的路径
    """
    path = os.path.normpath(path)
    _split_path = path.split('\\')
    path_reversed = ' \\ '.join(_split_path[::-1])
    path_reversed = os.path.normpath(path_reversed)
    return path_reversed


def _get_parent_dirpaths(path: str) -> list:
    """获取传入路径的所有上级目录路径
    :param path: 文件/文件夹路径
    :return: 所有上级目录列表，层级越高，顺序越前"""
    parent_dirs = []

    while True:
        parent_dirpath, filename = os.path.split(path)
        if filename:
            parent_dirs.append(parent_dirpath)
        else:
            break

        path = parent_dirpath

    # 反转列表顺序，使得越上级目录排在越前面
    parent_dirs = parent_dirs[::-1]

    return parent_dirs


def _remove_subpaths(paths: list):
    """剔除传入路径列表中的子路径，仅保留最上级路径"""
    paths = [os.path.normpath(path) for path in paths]
    paths = natsort.os_sorted(paths)  # 排序路径
    subpaths = []
    for path in paths:
        paths_split_parents = _get_parent_dirpaths(path)
        # 判断其父路径是否与原始列表存在交集，如果存在则说明其是子路径
        _s1 = set(paths)
        _s2 = set(paths_split_parents)
        if _s1 & _s2:
            subpaths.append(path)

    return [i for i in paths if i not in subpaths]


def _is_subpath(parent_path: str, child_path: str) -> bool:
    """判断一个路径是否为另一个路径的子路径"""
    # 获取绝对路径并规范化
    parent = os.path.abspath(os.path.normpath(parent_path))
    child = os.path.abspath(os.path.normpath(child_path))

    # 比较公共前缀
    common_prefix = os.path.commonpath([parent, child])
    return common_prefix == parent


"""----------调用函数----------"""


def split_path(path: str) -> Tuple[str, str, str]:
    """拆分路径为父目录路径，文件名（不含文件扩展名），文件扩展名
    :param path: 需要拆分的路径
    :return: 父目录路径，文件名（不含文件扩展名），文件扩展名"""
    return split_path(path)


def reverse_path(path: str) -> str:
    """反转路径层级，从后往前排列目录层级
    :param path: 需要反转的路径
    :return: 反转后的路径
    """
    return _reverse_path(path)


def get_parent_dirpaths(path: str) -> list:
    """获取传入路径的所有上级目录路径
    :param path: 文件/文件夹路径
    :return: 所有上级目录列表，层级越高，顺序越前"""
    return _get_parent_dirpaths(path)


def remove_subpaths(paths: list):
    """剔除传入路径列表中的子路径，仅保留最上级路径"""
    return _remove_subpaths(paths)


def is_subpath(parent_path: str, child_path: str) -> bool:
    """判断一个路径是否为另一个路径的子路径"""
    return _is_subpath(parent_path, child_path)
